#ifndef _INFRARED_C
#define _INFRARED_C

void ProcessSample( uint const _sample );
unsigned int s_samplesIN = 0;
unsigned int s_samplesOUT = 0;
unsigned int s_samples[MAX_SAMPLES];
int s_tarAtRise;
void GotCode( unsigned int _code );

// --------------------------------------------

#define PREFIX_SIZE 4

unsigned char s_state = 0;
unsigned int s_code = 0;
unsigned int s_bit = 1;
uint s_lastActivity = 0;

inline void _IR_Reset()
{
  s_state = 0;
  s_code = 0;
  s_bit = 1;
}

void IR_Init()
{
  P1DIR &= ~(1<<IR_PORT);
  
  s_lastActivity = TAR;
  s_tarAtRise = TAR;  
  _IR_Reset();
}

void IR_Advance()
{
  if( s_samplesIN == s_samplesOUT )
  {
    if( s_state && ((TAR - s_lastActivity) > 0x7000 ) )
    {
      _IR_Reset();
    }
    return;
  }
  
  s_lastActivity = TAR;
  uint _sample = s_samples[s_samplesOUT];
  s_samplesOUT = (s_samplesOUT+1) & (MAX_SAMPLES-1);
  GCC_BARRIER;

  unsigned char const state = s_state++;
  if( !state )
  {
    return;
  }
    
  if( state == 1 )
  {    
    if( ( _sample < 0x600) || ( _sample > 0x700) )
    {
      _IR_Reset();
    }
    return;
  }
  
  if( (_sample < 0x100)
      || (_sample > 0x580)
      || ((_sample > 0x200) && (_sample < 0x400))
    )
  {
    _IR_Reset();
    return;
  }
  
  unsigned char const bit = ( _sample < 0x300 );
  
  if( (!bit && ( state == 2 ))
      || (bit && (state == 3)) 
    )
  {
    _IR_Reset();
  }
  if( state <= 3 ) return;
  
  TransmitHex( _sample ); 
  TransmitString(_T(" "));
  GCC_BARRIER;
  if( bit )
  {
    s_code |= s_bit;
  }
  GCC_BARRIER;
  s_bit <<= 1;
  GCC_BARRIER;
  
  if( state == 0x11 )
  {
    GotCode( s_code );
    _IR_Reset();
  }
}

// ---------------------------------------------------------

forceinline void IR_Edge( bool const _edge )
{
  if( _edge )
  {
    s_samples[s_samplesIN] = TAR - s_tarAtRise;
    GCC_BARRIER;
    s_samplesIN = (s_samplesIN + 1) & (MAX_SAMPLES-1);
  }
  else
  {
    s_tarAtRise = TAR;
  }
}

#endif
