#include <io.h>
#include <signal.h>

// ------------------------------

typedef unsigned char bool;
typedef unsigned int uint;
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;
#define false (0)
#define true (1)
#define _T(x) (unsigned char const *)x
#define NULL (0)

#define noinline __attribute__((noinline))
#define forceinline __attribute__((always_inline))
#define GCC_BARRIER asm("":::"memory");

// ------------------------------

#define UART_OUTPUT (1)
#include "uart.c"

#define IR_PORT (5)
#define MAX_SAMPLES (16)
#include "infrared.c"

int main(void)
{
  WDTCTL = WDTPW + WDTHOLD; // Stop WDT
  
  P1SEL = 0;
  P1DIR = ~0;
  P1OUT = 0;
  P1REN = 0;
  P1IES = 0;
  P1IE = 0;

  IR_Init();
  InitUART();
  
  __bis_SR_register(GIE); // interrupts enabled  
  
  {
    unsigned int i;
    for( i = 0; i < 65534; i++ ) GCC_BARRIER;
  }
  
  P1IFG = 0;  
  P1IE |= (1<<IR_PORT);    // Enable interrupts for P1.IR_PORT
  
  GCC_BARRIER;

  
  TransmitString(_T("OK\r\n\r\n"));
  
  while(1)
  {
    GCC_BARRIER;
    IR_Advance();
  }
}

// ------

void GotCode( unsigned int _code )
{
  TransmitHex(_code);
  TransmitString(_T(" OK\r\n"));
  
  unsigned int i;
  switch(_code)
  {
    case 0x12df:
      // volume up
      {
        P1OUT |= (1<<7);
        for( i = 0; i < 1000; i++ )
          GCC_BARRIER;
        P1OUT &= ~(1<<7);
      }
      break;
     
    case 0x17df:
      // volume down
      {
        P1OUT |= (1<<6);
        for( i = 0; i < 1000; i++ )
          GCC_BARRIER;
        P1OUT &= ~(1<<6);
      }
      break;
      
    default: 
      break;
  }
}

// ------
interrupt(PORT1_VECTOR) PORT1_ISR(void)
{
  P1IFG = 0; // don't interrupt again...
  P1IES ^= (1<<IR_PORT); // ...unless it's for the next edge

  IR_Edge( (P1IES & (1<<IR_PORT)) == 0 );
}
