#ifndef _UART_C
#define _UART_C
#ifndef UART_OUTPUT
#define UART_OUTPUT (1)
#endif
#if UART_OUTPUT

#define BIT_TIME 104 // == 1MHz/9600
#define OUTMOD5 (5<<5)

unsigned char const c_digits[16] = { 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 'A', 'B', 'C', 'D', 'E', 'F' };

// Function Transmits Character from TXByte
unsigned int s_TxByte = 0x201;
unsigned int s_TxBit = 1;
unsigned char const c_zero[2] = { 0, 0 };
unsigned char const *s_TxPos = c_zero;

inline void InitUART()
{
  BCSCTL1 = CALBC1_1MHZ; // Set range
  DCOCTL = CALDCO_1MHZ; // SMCLK = DCO = 1MHz    
  TACTL = TASSEL_2 + MC_2; // SMCLK, continuous mode
  
  P1DIR |= (1<<1);
  P1DIR &= ~(1<<2);
  P1SEL |= (1<<1);
  P1OUT |= (1<<1);
  
  // reset serial connection: transmit a zero
  // globals already configured for this
  // so we just need to set the timer, enable interrupts and let rip
  CCR0 = TAR + BIT_TIME;
  CCTL0 = OUTMOD5 | CCIS0 | CCIE;  
}

noinline void TransmitString( unsigned char const *_s )
{
  if(!*_s)
  {
    return;
  }
  
  while ( CCTL0 & CCIE );

  s_TxBit = 2; // first start bit is sent here
  s_TxByte = (((*_s)|0x100)<<1);
  s_TxPos = _s + 1;
  GCC_BARRIER;
  CCR0 = TAR + BIT_TIME; // Initialize compare register
  CCTL0 = OUTMOD5 | CCIS0 | CCIE; // enable interrupts, queue start bit
}

unsigned char s_TxHex [] = { 0, 0, 0, 0, 0 };

void TransmitHex( unsigned int _hex )
{
  while ( CCTL0 & CCIE );
  s_TxHex[3] = c_digits[_hex & 0xF];
  s_TxHex[1] = c_digits[(_hex >> 8) & 0xF];
  _hex >>= 4;
  s_TxHex[2] = c_digits[_hex & 0xF];
  s_TxHex[0] = c_digits[(_hex >> 8) & 0xF];
  TransmitString( s_TxHex );
}

// Timer A0 interrupt service routine
interrupt(TIMERA0_VECTOR) Timer_A (void)
{
    CCR0 += BIT_TIME; // Add Offset to CCR0

    // work out what the next bit to be sent should be
    CCTL0 |= (1<<7); // assume logic 0 initially
    if( s_TxByte & s_TxBit )
    {
      CCTL0 &= ~(1<<7); // nope, sending a 1
    }
    GCC_BARRIER;
    
    if( !((s_TxBit <<= 1) & 0x400) )
    {
      return;
    }
    
    // any more data?
    if((s_TxByte = *s_TxPos)) // nb. assignment also used as test for zero
    {
       s_TxByte = ((s_TxByte|0x100)<<1);
       s_TxBit = 1;
       ++s_TxPos;
       return;
    }
    
    // all bits TXed
    CCTL0 &= ~CCIE; // Disable interrupt
}

#else

inline void InitUART() {};
inline void TransmitString( unsigned char const *_s ) {};
inline void TransmitHex( unsigned int _hex ) {};

#endif
#endif
